import * as hast from 'hast';
import * as hast_util_to_html_lib_types from 'hast-util-to-html/lib/types';
import { Element, Parent } from 'hast-util-to-html/lib/types';
export { Parent } from 'hast-util-to-html/lib/types';
import { IShikiTheme } from 'shiki';
import { ColorInput } from '@ctrl/tinycolor';

/**
 * A union of VS Code colors that we know the default values for. The default values are required
 * because VS Code themes do not need to define all colors, only the ones they want to change.
 *
 * This is not an exhaustive list of all VS Code colors and does not need to be. If a color is
 * missing here, it can still be set by themes and used by plugins - it will just have no defaults.
 */
type VSCodeDefaultColorKey = 'focusBorder' | 'foreground' | 'disabledForeground' | 'descriptionForeground' | 'errorForeground' | 'icon.foreground' | 'contrastActiveBorder' | 'contrastBorder' | 'textBlockQuote.background' | 'textBlockQuote.border' | 'textCodeBlock.background' | 'textLink.activeForeground' | 'textLink.foreground' | 'textPreformat.foreground' | 'textSeparator.foreground' | 'editor.background' | 'editor.foreground' | 'editorLineNumber.foreground' | 'editorLineNumber.activeForeground' | 'editorActiveLineNumber.foreground' | 'editor.selectionBackground' | 'editor.inactiveSelectionBackground' | 'editor.selectionHighlightBackground' | 'editorError.foreground' | 'editorWarning.foreground' | 'editorInfo.foreground' | 'editorHint.foreground' | 'problemsErrorIcon.foreground' | 'problemsWarningIcon.foreground' | 'problemsInfoIcon.foreground' | 'editor.findMatchBackground' | 'editor.findMatchHighlightBackground' | 'editor.findRangeHighlightBackground' | 'editorLink.activeForeground' | 'editorLightBulb.foreground' | 'editorLightBulbAutoFix.foreground' | 'diffEditor.insertedTextBackground' | 'diffEditor.insertedTextBorder' | 'diffEditor.removedTextBackground' | 'diffEditor.removedTextBorder' | 'diffEditor.insertedLineBackground' | 'diffEditor.removedLineBackground' | 'editorStickyScroll.background' | 'editorStickyScrollHover.background' | 'editorInlayHint.background' | 'editorInlayHint.foreground' | 'editorInlayHint.typeForeground' | 'editorInlayHint.typeBackground' | 'editorInlayHint.parameterForeground' | 'editorInlayHint.parameterBackground' | 'editorPane.background' | 'editorGroup.emptyBackground' | 'editorGroup.focusedEmptyBorder' | 'editorGroupHeader.tabsBackground' | 'editorGroupHeader.tabsBorder' | 'editorGroupHeader.noTabsBackground' | 'editorGroupHeader.border' | 'editorGroup.border' | 'editorGroup.dropBackground' | 'editorGroup.dropIntoPromptForeground' | 'editorGroup.dropIntoPromptBackground' | 'editorGroup.dropIntoPromptBorder' | 'sideBySideEditor.horizontalBorder' | 'sideBySideEditor.verticalBorder' | 'scrollbar.shadow' | 'scrollbarSlider.background' | 'scrollbarSlider.hoverBackground' | 'scrollbarSlider.activeBackground' | 'panel.background' | 'panel.border' | 'panelTitle.activeBorder' | 'panelTitle.activeForeground' | 'panelTitle.inactiveForeground' | 'panelSectionHeader.background' | 'terminal.background' | 'widget.shadow' | 'editorWidget.background' | 'editorWidget.foreground' | 'editorWidget.border' | 'quickInput.background' | 'quickInput.foreground' | 'quickInputTitle.background' | 'pickerGroup.foreground' | 'pickerGroup.border' | 'editor.hoverHighlightBackground' | 'editorHoverWidget.background' | 'editorHoverWidget.foreground' | 'editorHoverWidget.border' | 'editorHoverWidget.statusBarBackground' | 'titleBar.activeBackground' | 'titleBar.activeForeground' | 'titleBar.inactiveBackground' | 'titleBar.inactiveForeground' | 'titleBar.border' | 'toolbar.hoverBackground' | 'toolbar.activeBackground' | 'tab.activeBackground' | 'tab.unfocusedActiveBackground' | 'tab.inactiveBackground' | 'tab.unfocusedInactiveBackground' | 'tab.activeForeground' | 'tab.inactiveForeground' | 'tab.unfocusedActiveForeground' | 'tab.unfocusedInactiveForeground' | 'tab.hoverBackground' | 'tab.unfocusedHoverBackground' | 'tab.hoverForeground' | 'tab.unfocusedHoverForeground' | 'tab.border' | 'tab.lastPinnedBorder' | 'tab.activeBorder' | 'tab.unfocusedActiveBorder' | 'tab.activeBorderTop' | 'tab.unfocusedActiveBorderTop' | 'tab.hoverBorder' | 'tab.unfocusedHoverBorder' | 'tab.activeModifiedBorder' | 'tab.inactiveModifiedBorder' | 'tab.unfocusedActiveModifiedBorder' | 'tab.unfocusedInactiveModifiedBorder' | 'badge.background' | 'badge.foreground' | 'button.background' | 'button.foreground' | 'button.border' | 'button.separator' | 'button.hoverBackground' | 'button.secondaryBackground' | 'button.secondaryForeground' | 'button.secondaryHoverBackground' | 'dropdown.background' | 'dropdown.foreground' | 'dropdown.border' | 'list.activeSelectionBackground' | 'list.activeSelectionForeground' | 'tree.indentGuidesStroke' | 'input.background' | 'input.foreground' | 'input.placeholderForeground' | 'inputOption.activeBorder' | 'inputOption.hoverBackground' | 'inputOption.activeBackground' | 'inputOption.activeForeground' | 'inputValidation.infoBackground' | 'inputValidation.infoBorder' | 'inputValidation.warningBackground' | 'inputValidation.warningBorder' | 'inputValidation.errorBackground' | 'inputValidation.errorBorder' | 'keybindingLabel.background' | 'keybindingLabel.foreground' | 'keybindingLabel.border' | 'keybindingLabel.bottomBorder' | 'menu.foreground' | 'menu.background' | 'menu.selectionForeground' | 'menu.selectionBackground' | 'menu.separatorBackground' | 'editor.snippetTabstopHighlightBackground' | 'editor.snippetFinalTabstopHighlightBorder';
type VSCodeThemeType = 'dark' | 'light';
type VSCodeWorkbenchColors = {
    [key in VSCodeDefaultColorKey]: string;
} & {
    [key: string]: string;
};

declare class ExpressiveCodeTheme implements Omit<IShikiTheme, 'type' | 'colors'> {
    name: string;
    type: VSCodeThemeType;
    colors: VSCodeWorkbenchColors;
    fg: string;
    bg: string;
    semanticHighlighting: boolean;
    tokenColors: unknown;
    settings: IShikiTheme['settings'];
    /**
     * Loads the given theme for use with Expressive Code. Supports both Shiki and VS Code themes.
     *
     * Note: To save on bundle size, this constructor does not support loading themes
     * bundled with Shiki by name (e.g. `dracula`). Instead, import Shiki's `loadTheme`
     * function yourself, use it to load its bundled theme (e.g. `themes/dracula.json`),
     * and pass the result to this constructor.
     */
    constructor(theme: Partial<ExpressiveCodeTheme> | (Partial<IShikiTheme> & {
        semanticHighlighting?: boolean | undefined;
        tokenColors?: unknown | undefined;
    }));
    /**
     * Attempts to parse the given JSON string as a theme.
     *
     * As some themes follow the JSONC format and may contain comments and trailing commas,
     * this method will attempt to strip them before parsing the result.
     */
    static fromJSONString(json: string): ExpressiveCodeTheme;
}

type CoreStyleSettings = 'borderRadius' | 'borderWidth' | 'borderColor' | 'codeFontFamily' | 'codeFontSize' | 'codeFontWeight' | 'codeLineHeight' | 'codePaddingBlock' | 'codePaddingInline' | 'codeBackground' | 'codeForeground' | 'codeSelectionBackground' | 'uiFontFamily' | 'uiFontSize' | 'uiFontWeight' | 'uiLineHeight' | 'uiPaddingBlock' | 'uiPaddingInline' | 'uiSelectionBackground' | 'uiSelectionForeground' | 'focusBorder' | 'scrollbarThumbColor' | 'scrollbarThumbHoverColor';
declare const coreStyleSettings: StyleSettings<CoreStyleSettings>;
type ResolvedCoreStyles = ResolvedStyleSettings<CoreStyleSettings>;
declare const codeLineClass = "ec-line";
declare function getCoreBaseStyles(options: {
    theme: ExpressiveCodeTheme;
    coreStyles: ResolvedCoreStyles;
    useThemedScrollbars: boolean;
    useThemedSelectionColors: boolean;
}): string;

/**
 * Represents a strongly typed set of style settings that can be used in plugins.
 *
 * The setting property names are taken from the default styles passed to the constructor.
 * The values can either be plain strings or resolver functions that return a string.
 *
 * The passed default style settings are made available as a public property.
 *
 * Call the `resolve()` method to get a resolved style settings object
 * that can be used when generating CSS styles.
 *
 * @example
 * const framesStyleSettings = new StyleSettings({
 *   fontFamily: 'sans-serif',
 *   fontSize: '1rem',
 *   titleBarForeground: ({ theme }) => theme.colors['editor.foreground'],
 * })
 * // ↓↓↓
 * framesStyleSettings.defaultSettings.fontFamily         // 'sans-serif'
 * framesStyleSettings.defaultSettings.fontSize           // '1rem'
 * framesStyleSettings.defaultSettings.titleBarForeground // ({ theme }) => theme.colors['editor.foreground']
 */
declare class StyleSettings<T extends string> {
    readonly defaultSettings: UnresolvedStyleSettings<T>;
    constructor(defaultSettings: UnresolvedStyleSettings<T>);
    /**
     * Takes an object with the properties `theme` and optional `styleOverrides` as input.
     * If any `styleOverrides` are defined, they will be used to override the default styles
     * during resolving.
     *
     * If any of the default styles or style overrides are not strings, but resolver functions,
     * they will be called with the given `theme` object as a property and are expected to
     * return a string which is used as the resolved property value.
     *
     * @example
     * // (Continuing with the instance created in the class usage example)
     * framesStyleSettings.resolve({
     *   theme: new ExpressiveCodeTheme({ ... }),
     *   styleOverrides: {
     *     fontSize: '2rem',
     *     titleBarForeground: ({ theme }) => theme.colors['menu.foreground'],
     *   }
     * })
     * // ↓↓↓
     * {
     *   fontFamily: 'sans-serif',     // default style (as passed to the constructor earlier)
     *   fontSize: '2rem',             // style override (plain string)
     *   titleBarForeground: '#f0f0f0' // style override (return value of resolver function)
     * }
     */
    resolve({ theme, coreStyles, styleOverrides, }: {
        theme: ExpressiveCodeTheme;
        coreStyles: ResolvedCoreStyles;
        styleOverrides?: Partial<UnresolvedStyleSettings<T>> | undefined;
    }): ResolvedStyleSettings<T>;
}
type ColorDefinition = string | [dark: string, light: string];
type CoreStyleResolverFn<T extends string> = ({ theme, resolveSetting, }: {
    theme: ExpressiveCodeTheme;
    resolveSetting: (propertyName: T) => string;
}) => ColorDefinition;
type StyleResolverFn<T extends string> = ({ theme, coreStyles, resolveSetting, }: {
    theme: ExpressiveCodeTheme;
    coreStyles: ResolvedCoreStyles;
    resolveSetting: (propertyName: T) => string;
}) => ColorDefinition;
type BaseStylesResolverFn = ({ theme, coreStyles }: {
    theme: ExpressiveCodeTheme;
    coreStyles: ResolvedCoreStyles;
}) => string | Promise<string>;
type UnresolvedCoreStyleSettings<T extends string> = {
    [K in T]: ColorDefinition | CoreStyleResolverFn<T>;
};
type UnresolvedStyleSettings<T extends string> = {
    [K in T]: ColorDefinition | StyleResolverFn<T>;
};
type ResolvedStyleSettings<T extends string> = {
    [K in T]: string;
};

type PluginStyles = {
    pluginName: string;
    styles: string;
};

type RenderInput = ExpressiveCodeBlockOptions | ExpressiveCodeBlock | (ExpressiveCodeBlockOptions | ExpressiveCodeBlock)[];
interface RenderOptions {
    /**
     * An optional handler function that can initialize plugin data for the
     * code block group before processing starts.
     *
     * Plugins can provide access to their data by exporting a const
     * set to a `new AttachedPluginData(...)` instance (e.g. `myPluginData`).
     *
     * You can then import the const and set `onInitGroup` to a function that
     * calls `myPluginData.setFor(group, { ...data... })`.
     */
    onInitGroup?: ((groupContents: GroupContents) => void) | undefined;
}
type GroupContents = readonly {
    codeBlock: ExpressiveCodeBlock;
}[];
type RenderedGroupContents = readonly {
    codeBlock: ExpressiveCodeBlock;
    renderedBlockAst: Element;
}[];

interface ExpressiveCodeHookContext {
    codeBlock: ExpressiveCodeBlock;
    groupContents: GroupContents;
    theme: ExpressiveCodeTheme;
    locale: string;
    coreStyles: ResolvedCoreStyles;
    /**
     * Adds CSS styles to the document that contains the rendered code.
     *
     * All styles are scoped to Expressive Code by default, so they will not affect
     * the rest of the page. SASS-like nesting is supported. If you want to add global styles,
     * you can use the `@at-root` rule or target `:root`, `html` or `body` in your selectors.
     *
     * The engine's `render` function returns all added styles in a string array along with
     * the rendered group and block ASTs. The calling code must take care of actually adding
     * these styles to the page. For example, it could insert them into a `<style>` element
     * before the rendered code block.
     *
     * **Note for integration authors:** If you are rendering multiple code block groups on the
     * same HTML page, you should deduplicate the returned styles at the page level.
     * Expressive Code deduplicates styles added to the same group before returning them,
     * but is not aware which styles are already present on the page.
     *
     * **Note for plugin authors:** If you are adding the same styles to every block,
     * consider using the `baseStyles` property of the plugin instead. This allows integrations
     * to optionally extract these styles into a separate CSS file.
     */
    addStyles: (css: string) => void;
}
interface PostprocessRenderedLineContext extends ExpressiveCodeHookContext {
    line: ExpressiveCodeLine;
    lineIndex: number;
    renderData: {
        lineAst: Element;
    };
}
interface PostprocessRenderedBlockContext extends ExpressiveCodeHookContext {
    renderData: {
        blockAst: Element;
    };
}
interface PostprocessRenderedBlockGroupContext {
    renderedGroupContents: RenderedGroupContents;
    pluginStyles: PluginStyles[];
    addStyles: (css: string) => void;
    renderData: {
        groupAst: Parent;
    };
}
type ExpressiveCodeHook<ContextType = ExpressiveCodeHookContext> = (context: ContextType) => void | Promise<void>;
interface ExpressiveCodePluginHooks_BeforeRendering {
    /**
     * Allows preprocessing the meta string and the language before any plugins can
     * modify the code.
     *
     * Plugins are expected to use this hook to remove any of their syntax from the meta string.
     * Removed information can either be stored internally or used to create annotations.
     *
     * As the code still matches the plaintext in the containing Markdown/MDX document at this
     * point, this hook can be used to apply annotations by line numbers.
     */
    preprocessMetadata?: ExpressiveCodeHook | undefined;
    /**
     * Allows preprocessing the code before any language-specific hooks are run.
     *
     * Plugins are expected to use this hook to remove any of their syntax that could disturb
     * annotation plugins like syntax highlighters. Removed information can either be stored
     * internally or used to create annotations.
     *
     * Plugins can also use this hook to insert new code, e.g. to add type information for
     * syntax highlighters, or to provide functionality to include external files into the
     * code block.
     */
    preprocessCode?: ExpressiveCodeHook | undefined;
    /**
     * Allows analyzing the preprocessed code and collecting language-specific syntax annotations.
     *
     * This hook is used by syntax highlighting plugins to run the code through `Shiki` or
     * `Shiki-Twoslash` and to create annotations from their highlighted tokens.
     *
     * These annotations are then available to the following hooks and will be used during
     * rendering.
     */
    performSyntaxAnalysis?: ExpressiveCodeHook | undefined;
    /**
     * Allows postprocessing the code plaintext after collecting syntax annotations.
     *
     * Plugins are expected to use this hook to remove any parts from the code
     * that should not be contained in the output. For example, if a plugin added declarations
     * or type information during the `preprocessCode` hook to provide information to the
     * syntax highlighter, the declarations could now be removed again.
     *
     * After this hook has finished processing, the plaintext of all code lines becomes read-only.
     */
    postprocessAnalyzedCode?: ExpressiveCodeHook | undefined;
    /**
     * Allows annotating the final code plaintext.
     *
     * As the code is read-only at this point, plugins can use this hook to create annotations
     * on lines or inline ranges matching a specific search term.
     */
    annotateCode?: ExpressiveCodeHook | undefined;
    /**
     * Allows applying final changes to annotations before rendering.
     *
     * After this hook has finished processing, all annotations become read-only.
     */
    postprocessAnnotations?: ExpressiveCodeHook | undefined;
}
interface ExpressiveCodePluginHooks_Rendering {
    /**
     * Allows editing the AST of a single line of code after all annotations were rendered.
     */
    postprocessRenderedLine?: ExpressiveCodeHook<PostprocessRenderedLineContext> | undefined;
    /**
     * Allows editing the AST of the entire code block after all annotations were rendered
     * and all lines were postprocessed.
     */
    postprocessRenderedBlock?: ExpressiveCodeHook<PostprocessRenderedBlockContext> | undefined;
    /**
     * Allows editing the ASTs of all code blocks that were rendered as part of the same group,
     * as well as the AST of the group root element that contains all group blocks.
     *
     * Groups are defined by the calling code. For example, a Remark plugin using Expressive Code
     * to render code blocks could provide authors with a way to group related code blocks together.
     *
     * This hook is used by the frames plugin to display multiple code blocks as editor file tabs.
     *
     * Note: Even if a code block is not part of any group, this hook will still be called.
     * Standalone code blocks are treated like a group containing only a single block.
     */
    postprocessRenderedBlockGroup?: ExpressiveCodeHook<PostprocessRenderedBlockGroupContext> | undefined;
}
interface ExpressiveCodePluginHooks extends ExpressiveCodePluginHooks_BeforeRendering, ExpressiveCodePluginHooks_Rendering {
}
type ExpressiveCodePluginHookName = keyof ExpressiveCodePluginHooks;
/**
 * Runs the given `runner` function for every hook that was registered by plugins
 * for the given hook type.
 *
 * The runner function is called with an object containing the hook name, the hook function
 * registered by the plugin, and the plugin that registered it.
 *
 * Errors occuring in the runner function are caught and rethrown with information about the
 * plugin and hook that caused the error.
 */
declare function runHooks<HookType extends keyof ExpressiveCodePluginHooks>(key: HookType, plugins: readonly ExpressiveCodePlugin[], runner: (hook: {
    hookName: HookType;
    hookFn: NonNullable<ExpressiveCodePluginHooks[HookType]>;
    plugin: ExpressiveCodePlugin;
}) => void | Promise<void>): Promise<void>;

type JsModulesResolverFn = ({ theme, coreStyles, configClassName, }: {
    theme: ExpressiveCodeTheme;
    coreStyles: ResolvedCoreStyles;
    configClassName: string;
}) => string[] | Promise<string[]>;
interface ExpressiveCodePlugin {
    name: string;
    /**
     * The CSS styles that should be added to every page containing code blocks.
     *
     * All styles are scoped to Expressive Code by default, so they will not affect
     * the rest of the page. SASS-like nesting is supported. If you want to add global styles,
     * you can use the `@at-root` rule or target `:root`, `html` or `body` in your selectors.
     *
     * The engine's `getBaseStyles` function goes through all registered plugins
     * and collects their base styles.
     *
     * If you provide a function instead of a string, it is called with an object argument
     * containing the current theme, the resolved core styles and the config-dependent
     * wrapper class name. It is expected to either return a string or a string promise.
     *
     * The calling code must take care of actually adding the collected styles to the page.
     * For example, it could create a site-wide CSS stylesheet from the base styles
     * and insert a link to it, or it could insert the base styles into a `<style>` element.
     */
    baseStyles?: string | BaseStylesResolverFn | undefined;
    /**
     * JavaScript modules (pure code without any wrapping `script` tags) that should be added
     * to every page containing code blocks.
     *
     * The engine's `getJsModules` function goes through all registered plugins,
     * collects their JS modules and deduplicates them.
     *
     * If you provide a function instead of a string, it is called with an object argument
     * containing the current theme, the resolved core styles and the config-dependent
     * wrapper class name. It is expected to either return or promise a string array.
     *
     * The calling code must take care of actually adding the collected scripts to the page.
     * For example, it could create site-wide JavaScript files from the returned modules
     * and refer to them in a script tag with `type="module"`, or it could insert them
     * into inline `<script type="module">` elements.
     */
    jsModules?: string[] | JsModulesResolverFn | undefined;
    hooks: ExpressiveCodePluginHooks;
}

interface ExpressiveCodeProcessingState {
    canEditCode: boolean;
    canEditMetadata: boolean;
    canEditAnnotations: boolean;
}

interface ExpressiveCodeBlockOptions {
    code: string;
    language: string;
    meta?: string | undefined;
    /**
     * The code block's locale (e.g. `en-US` or `de-DE`). This is used by plugins to display
     * localized strings depending on the language of the containing page.
     *
     * Integrations like `remark-expressive-code` support multi-language sites by allowing you
     * to provide custom logic to determine a block's locale (e.g. based on its parent document).
     *
     * If no locale is defined here, `ExpressiveCodeEngine` will render the code block
     * using the `defaultLocale` provided in its configuration.
     */
    locale?: string | undefined;
    /**
     * Optional data about the parent document the code block is located in.
     *
     * Integrations like `remark-expressive-code` can provide this information based on the
     * source document being processed. There may be cases where no document is available,
     * e.g. when the code block was created dynamically.
     */
    parentDocument?: {
        /**
         * The full path to the source file containing the code block.
         */
        sourceFilePath?: string | undefined;
    } | undefined;
}
declare class ExpressiveCodeBlock {
    #private;
    constructor(options: ExpressiveCodeBlockOptions);
    /**
     * This field exists to ensure that only actual class instances are accepted
     * as the type `ExpressiveCodeBlock` by TypeScript. Without this workaround,
     * plain objects with the same structure would be accepted, but fail at runtime.
     */
    private _requireInstance;
    get code(): string;
    get language(): string;
    set language(value: string);
    get meta(): string;
    set meta(value: string);
    get locale(): string | undefined;
    get parentDocument(): {
        /**
         * The full path to the source file containing the code block.
         */
        sourceFilePath?: string | undefined;
    } | undefined;
    get state(): ExpressiveCodeProcessingState | undefined;
    set state(value: ExpressiveCodeProcessingState | undefined);
    getLine(index: number): ExpressiveCodeLine | undefined;
    getLines(startIndex?: number, endIndex?: number): readonly ExpressiveCodeLine[];
    deleteLine(index: number): void;
    deleteLines(indices: number[]): void;
    insertLine(index: number, textLine: string): ExpressiveCodeLine;
    insertLines(index: number, textLines: string[]): ExpressiveCodeLine[];
}

declare class ExpressiveCodeLine {
    #private;
    constructor(text: string);
    get text(): string;
    get parent(): ExpressiveCodeBlock | undefined;
    set parent(value: ExpressiveCodeBlock | undefined);
    getAnnotations(): readonly ExpressiveCodeAnnotation[];
    addAnnotation(annotation: ExpressiveCodeAnnotation): void;
    deleteAnnotation(annotation: ExpressiveCodeAnnotation): void;
    editText(columnStart: number | undefined, columnEnd: number | undefined, newText: string): string;
}

type ExpressiveCodeInlineRange = {
    columnStart: number;
    columnEnd: number;
};

type AnnotationRenderOptions = {
    nodesToTransform: Parent[];
    line: ExpressiveCodeLine;
};
type AnnotationRenderPhase = 'earliest' | 'earlier' | 'normal' | 'later' | 'latest';
declare const AnnotationRenderPhaseOrder: AnnotationRenderPhase[];
declare function annotationSortFn(a: ExpressiveCodeAnnotation, b: ExpressiveCodeAnnotation): number;
type AnnotationBaseOptions = {
    inlineRange?: ExpressiveCodeInlineRange | undefined;
    renderPhase?: AnnotationRenderPhase | undefined;
};
declare abstract class ExpressiveCodeAnnotation {
    constructor({ inlineRange, renderPhase }: AnnotationBaseOptions);
    /**
     * Renders the annotation by transforming the provided nodes.
     *
     * This function will be called with an array of AST nodes to transform, and is expected
     * to return an array containing the same number of nodes.
     *
     * For example, you could use the `hastscript` library to wrap the received nodes
     * in HTML elements.
     */
    abstract render({ nodesToTransform, line }: AnnotationRenderOptions): Parent[];
    /**
     * An optional range of columns within the line that this annotation applies to.
     * If not provided, the annotation will apply to the entire line.
     */
    readonly inlineRange: ExpressiveCodeInlineRange | undefined;
    /**
     * Determines the phase in which this annotation should be rendered.
     * Rendering is done in phases, from `earliest` to `latest`.
     * Annotations with the same phase are rendered in the order they were added.
     *
     * The earlier an annotation is rendered, the more likely it is to be split, modified
     * or wrapped by later annotations. Syntax highlighting is rendered in the `earliest` phase
     * to allow other annotations to wrap and modify the highlighted code.
     *
     * The default phase is `normal`.
     */
    readonly renderPhase: AnnotationRenderPhase | undefined;
}
declare class InlineStyleAnnotation extends ExpressiveCodeAnnotation {
    color: string | undefined;
    italic: boolean;
    bold: boolean;
    underline: boolean;
    constructor({ color, italic, bold, underline, ...baseOptions }: {
        color?: string | undefined;
        italic?: boolean | undefined;
        bold?: boolean | undefined;
        underline?: boolean | undefined;
    } & AnnotationBaseOptions);
    render({ nodesToTransform }: AnnotationRenderOptions): hast.Element[];
}
declare function validateExpressiveCodeAnnotation(annotation: ExpressiveCodeAnnotation): void;

interface ExpressiveCodeEngineConfig {
    /**
     * The color theme that should be used when rendering.
     *
     * Defaults to the `github-dark` theme bundled with Shiki.
     */
    theme?: ExpressiveCodeTheme | undefined;
    /**
     * The locale that should be used for text content. Defaults to `en-US`.
     */
    defaultLocale?: string | undefined;
    /**
     * Whether the theme is allowed to style the scrollbars. Defaults to `true`.
     *
     * If set to `false`, scrollbars will be rendered using the browser's default style.
     *
     * Note that you can override the individual scrollbar colors defined by the theme
     * using the `styleOverrides` option.
     */
    useThemedScrollbars?: boolean | undefined;
    /**
     * Whether the theme is allowed to style selected text. Defaults to `true`.
     *
     * If set to `false`, selected text will be rendered using the browser's default style.
     *
     * Note that you can override the individual selection colors defined by the theme
     * using the `styleOverrides` option.
     */
    useThemedSelectionColors?: boolean | undefined;
    /**
     * An optional set of style overrides that can be used to customize the appearance of
     * the rendered code blocks without having to write custom CSS. You can customize core
     * colors, fonts, paddings and more.
     *
     * If any of the settings are not given, default values will be used or derived from the theme,
     * as seen in the exported `coreStyleSettings` object.
     *
     * **Tip:** If your site uses CSS variables for styling, you can also use these overrides
     * to replace any core style with a CSS variable reference, e.g. `var(--your-css-var)`.
     */
    styleOverrides?: Partial<UnresolvedCoreStyleSettings<CoreStyleSettings>> | undefined;
    /**
     * To add a plugin, import its initialization function and call it inside this array.
     *
     * If the plugin has any configuration options, you can pass them to the initialization
     * function as an object containing your desired property values.
     */
    plugins?: (ExpressiveCodePlugin | ExpressiveCodePlugin[])[] | undefined;
}
declare class ExpressiveCodeEngine {
    constructor(config: ExpressiveCodeEngineConfig);
    render(input: RenderInput, options?: RenderOptions): Promise<{
        renderedGroupAst: hast_util_to_html_lib_types.Parent;
        renderedGroupContents: RenderedGroupContents;
        styles: Set<string>;
    }>;
    getBaseStyles(): Promise<string>;
    /**
     * Returns an array of JavaScript modules (pure core without any wrapping `script` tags)
     * that should be added to every page containing code blocks.
     *
     * The contents are collected from the `jsModules` property of all registered plugins.
     * Any duplicates are removed.
     *
     * The calling code must take care of actually adding the collected scripts to the page.
     * For example, it could create site-wide JavaScript files from the returned modules
     * and refer to them in a script tag with `type="module"`, or it could insert them
     * into inline `<script type="module">` elements.
     */
    getJsModules(): Promise<string[]>;
    readonly theme: ExpressiveCodeTheme;
    readonly defaultLocale: string;
    readonly useThemedScrollbars: boolean;
    readonly useThemedSelectionColors: boolean;
    readonly styleOverrides: Partial<typeof coreStyleSettings.defaultSettings>;
    readonly coreStyles: ResolvedCoreStyles;
    readonly plugins: readonly ExpressiveCodePlugin[];
    /**
     * This class name is used by Expressive Code when rendering its wrapper element
     * around all code block groups.
     *
     * Its format is `ec.ec-<hash>`, where `<hash>` is calculated based on the config options
     * that were passed to the class constructor. This allows you to render multiple code blocks
     * with different configurations on the same page without having to worry about CSS conflicts.
     *
     * Non-global CSS styles returned by the `getBaseStyles` and `render` methods
     * are scoped automatically using this class name.
     *
     * **Note to website authors:** The default class `expressive-code` is also added to all
     * wrapper elements, providing you with a way to target all code blocks in CSS
     * regardless of the config options.
     */
    readonly configClassName: string;
}

type PluginDataTarget = ExpressiveCodeBlock | GroupContents | RenderedGroupContents;
declare class AttachedPluginData<PluginDataType> {
    private readonly dataStorage;
    private readonly getInitialValueFn;
    constructor(getInitialValueFn: () => PluginDataType);
    getOrCreateFor(target: PluginDataTarget): PluginDataType;
    setFor(target: PluginDataTarget, data: PluginDataType): void;
}

type KnownTextsByKey<T extends {
    [K: string]: string | (() => string);
}> = {
    [K in keyof T]: T[K];
};
declare class PluginTexts<Texts extends KnownTextsByKey<Texts>> {
    readonly defaultTexts: Texts;
    private readonly localizedTexts;
    private readonly overridesByLocale;
    constructor(defaultTexts: Texts);
    /**
     * Adds localized texts for a specific locale. You must provide a full set of localized texts
     * for the given locale.
     *
     * It is recommended to use two-letter language codes (e.g. `de`, `fr`, `es`) without region
     * codes to make your localized texts available to all users speaking the same language.
     * Region codes should only be added if regional differences must be taken into account.
     *
     * Plugin authors can use this to provide localized versions of their texts.
     * Users can also call this function to provide their own localizations.
     *
     * If you only want to customize a few texts of an existing localization,
     * have a look at `overrideTexts` instead.
     */
    addLocale(locale: string, localizedTexts: Texts): void;
    /**
     * Allows you to override any defined texts. This is useful if you want to customize a few
     * selected texts without having to provide a full set of localized texts.
     *
     * You can either override texts for a specific `locale`, or override the default texts
     * by setting `locale` to `undefined`.
     *
     * It is recommended to use two-letter language codes (e.g. `de`, `fr`, `es`) without region
     * codes to apply your overrides to all users speaking the same language.
     * Region codes should only be added if regional differences must be taken into account.
     */
    overrideTexts(locale: string | undefined, localeTextOverrides: Partial<Texts>): void;
    /**
     * Returns the best matching texts for the requested locale,
     * taking any available localized texts and overrides into account.
     *
     * Example for locale `de-DE`:
     * - If localized texts for `de-DE` are available, these will be returned.
     * - If `de-DE` is not available, but `de` is, these will be returned.
     * - As the final fallback, the default texts will be returned.
     */
    get(locale: string): Texts;
    private parseLocale;
    private getLocalizedTexts;
    private applyOverrides;
}

declare function addClass(node: Parent, className: string): void;

type LabColor = {
    l: number;
    a: number;
    b: number;
    alpha?: number | undefined;
};
type LchColor = {
    l: number;
    c: number;
    h: number;
    alpha?: number | undefined;
};

type ColorValue = ColorInput | LabColor | LchColor;
/**
 * Overrides the alpha value of a color with the given value.
 * Values should be between 0 and 1.
 */
declare function setAlpha(input: ColorValue, newAlpha: number): string;
/**
 * Multiplies the existing alpha value of a color with the given factor.
 * Automatically limits the resulting alpha value to the range 0 to 1.
 */
declare function multiplyAlpha(input: ColorValue, factor: number): string;
/**
 * Mixes a color with white or black to achieve the desired luminance.
 * Luminance values should be between 0 and 1.
 */
declare function setLuminance(input: ColorValue, targetLuminance: number): string;
/**
 * Lightens a color by the given amount.
 * Automatically limits the resulting lightness value to the range 0 to 1.
 */
declare function lighten(input: ColorValue, amount: number): string;
/**
 * Darkens a color by the given amount.
 * Automatically limits the resulting lightness value to the range 0 to 1.
 */
declare function darken(input: ColorValue, amount: number): string;
/**
 * Mixes the second color into the first color by the given amount.
 * Amount should be between 0 and 1.
 */
declare function mix(input: ColorValue, mixinInput: ColorValue, amount: number): string;
/**
 * Computes how the first color would look on top of the second color.
 */
declare function onBackground(input: ColorValue, background: ColorValue): string;
declare function getColorContrast(color1: ColorValue, color2: ColorValue): number;
declare function getColorContrastOnBackground(input: ColorValue, background: ColorValue): number;
/**
 * Modifies the luminance and/or the alpha value of a color to ensure its color contrast
 * on the given background color is within the given range.
 *
 * - If the contrast is too low, the luminance is either increased or decreased first,
 *   and then the alpha value is increased (if required).
 * - If the contrast is too high, only the alpha value is decreased.
 *
 * If the target contrast cannot be reached, the function will try to get as close as possible.
 */
declare function ensureColorContrastOnBackground(input: ColorValue, background: ColorValue, minContrast?: number, maxContrast?: number): string;
declare function changeLuminanceToReachColorContrast(input1: ColorValue, input2: ColorValue, minContrast?: number): string;
declare function changeAlphaToReachColorContrast(input: ColorValue, background: ColorValue, minContrast?: number, maxContrast?: number): string;
declare function toHexColor(input: ColorValue): string;
declare function toRgbaString(input: ColorValue): string;

/**
 * Returns a JSON-like string representation of the given object that is stable,
 * meaning that keys are sorted alphabetically. This causes objects with the same keys and values
 * to always have the same string representation.
 *
 * Circular references are handled by replacing them with the string `[Circular]`.
 *
 * Functions are replaced with the string `[Function]`, unless the `includeFunctionContents`
 * option is set to `true`.
 */
declare function stableStringify(obj: unknown, options?: {
    includeFunctionContents?: boolean | undefined;
}): string;
/**
 * Returns a simple hash of the given object.
 *
 * The hash is stable, meaning that if the object has the same keys and values (in any order),
 * the hash will be the same. The hash is not cryptographically secure, but uses the simple
 * and fast `djb2a` algorithm, which is known to produce few collisions.
 */
declare function getStableObjectHash(obj: unknown, options?: {
    includeFunctionContents?: boolean | undefined;
    hashLength?: number | undefined;
}): string;

type ReplaceDelimitedValuesMatch = {
    fullMatch: string;
    key: string;
    value: string;
    valueStartDelimiter: string;
    valueEndDelimiter: string;
};
type ReplaceDelimitedValuesSyntax = {
    /**
     * An array defining all strings allowed as value delimiters, which indicate
     * the start & end of a value inside the processed string.
     *
     * By default, the same delimiter must be used to start and end a value,
     * so `['"', "'"]` would detect values delimited with `"double"` and `'single'`,
     * but not `"mixed'` quotes.
     *
     * If you want to use a different delimiter to end a value,
     * specify the start and end delimiter pair separated by three dots.
     * For example, `['{...}']` would detect values delimited `{like this}`.
     */
    valueDelimiters: string[];
    keyValueSeparator: string;
};
declare function replaceDelimitedValues(
/** The input string. */
input: string, 
/**
 * This function gets called on every match.
 *
 * It is expected to return the replacement string for the match.
 */
replacer: (match: ReplaceDelimitedValuesMatch) => string, 
/**
 * Optional syntax settings that determine how values are delimited,
 * and how keys and values are separated from each other.
 */
syntax?: ReplaceDelimitedValuesSyntax): string;
/**
 * Retrieves all group indices from the given RegExp match. Group indices are ranges
 * defined by start & end positions. The first group index refers to the full match,
 * and the following indices to RegExp capture groups (if any).
 *
 * If the RegExp flag `d` was enabled (and supported), it returns the native group indices.
 *
 * Otherwise, it uses fallback logic to manually search for the group contents inside the
 * full match. Note that this can be wrong if a group's contents can be found multiple times
 * inside the full match, but that's probably a rare case and still better than failing.
 */
declare function getGroupIndicesFromRegExpMatch(match: RegExpMatchArray): ([start: number, end: number] | null)[];

export { AnnotationBaseOptions, AnnotationRenderOptions, AnnotationRenderPhase, AnnotationRenderPhaseOrder, AttachedPluginData, BaseStylesResolverFn, ColorDefinition, ColorValue, CoreStyleResolverFn, CoreStyleSettings, ExpressiveCodeAnnotation, ExpressiveCodeBlock, ExpressiveCodeBlockOptions, ExpressiveCodeEngine, ExpressiveCodeEngineConfig, ExpressiveCodeHook, ExpressiveCodeHookContext, ExpressiveCodeInlineRange, ExpressiveCodeLine, ExpressiveCodePlugin, ExpressiveCodePluginHookName, ExpressiveCodePluginHooks, ExpressiveCodePluginHooks_BeforeRendering, ExpressiveCodePluginHooks_Rendering, ExpressiveCodeTheme, InlineStyleAnnotation, JsModulesResolverFn, PluginDataTarget, PluginTexts, PostprocessRenderedBlockContext, PostprocessRenderedBlockGroupContext, PostprocessRenderedLineContext, ReplaceDelimitedValuesMatch, ReplaceDelimitedValuesSyntax, ResolvedCoreStyles, ResolvedStyleSettings, StyleResolverFn, StyleSettings, UnresolvedCoreStyleSettings, UnresolvedStyleSettings, addClass, annotationSortFn, changeAlphaToReachColorContrast, changeLuminanceToReachColorContrast, codeLineClass, coreStyleSettings, darken, ensureColorContrastOnBackground, getColorContrast, getColorContrastOnBackground, getCoreBaseStyles, getGroupIndicesFromRegExpMatch, getStableObjectHash, lighten, mix, multiplyAlpha, onBackground, replaceDelimitedValues, runHooks, setAlpha, setLuminance, stableStringify, toHexColor, toRgbaString, validateExpressiveCodeAnnotation };
